# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/5/20] ------------------------- #
# --------------------------------------------- #

module Strum::Internal::Events
  ##
  # Raised when Component executes a task.
  class ComponentEvent < Event
    include Strum::Internal::Container
    ##
    # Creates a new ComponentEvent
    def initialize(params = {}, block)
      raise Strum::Errors::NilComponent unless params[:Component]

      self[:Component] = params[:Component]
      super(params, block)
    end
  end
  ##
  # Raised when a Component hooks into a receiver.
  class ComponentHookEvent < ComponentEvent; end
  ##
  # Raised when a Component is initialized
  class ComponentInitializationEvent < ComponentEvent; end
  ##
  # Raised when a Component reports to the logger object.
  class ComponentReportEvent < ComponentEvent; end
  ##
  # Raised when a Component executes an Asynchronous event.
  class ComponentAsyncEvent < ComponentEvent; end
  ##
  # Raised when a Component is tranquilized
  class ComponentTranqEvent < ComponentEvent; end
end