# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/5/20] ------------------------- #
# --------------------------------------------- #

module Strum::Internal::Events
  ##
  # Raised when an a Socket triggers an event.
  class SocketEvent < Event
    ##
    # Raised when a Socket performs an action.
    def initialize(params = {}, block)
      if params[:ip]
        self[:Socket] = Strum::Server::Session::Manager.instance[:Sessions]
      end

      self[:Socket] = params[:socket]
      super(params, block)
    end
  end
  ##
  # Raised when a Socket is received by a SessionManager object.
  class SocketReceiveEvent < SocketEvent; end
  ##
  # Raised when a Socket times out
  class SocketTimeoutEvent < SocketEvent; end
  ##
  # Raised when a Socket is forced closed.
  class SocketForceCloseEvent < SocketEvent; end
  ##
  # Raised when a Socket closes naturally
  class SocketClosedEvent < SocketEvent; end
  ##
  # Raised when a Socket connects and triggers a delay violation.
  class SocketDelayViolationEvent < SocketEvent; end
  ##
  # Raised when a Socket is throttled.
  class SocketThrottleEvent < SocketEvent; end
  ##
  # Raised when a Socket cannot be received because the Session List has reached capacity.
  class SocketListOverflow < SocketEvent; end
end