# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/3/20] ------------------------- #
# --------------------------------------------- #

module Strum::Server
  class Interceptor
    include Singleton
    include Strum::Internal::Component
    ##
    # Settings for this manager
    Settings = Struct.new(
        :MAX_SIZE,
        :THROTTLE_COUNT,
        :DELAY_TIME)
    ##
    # Create a Gateway object and spawn an endpoint for incoming connections.
    # @param settings [Hash] Settings for this interceptor.
    def initialize(settings = {})
      self[:Settings] = Settings.new(
          ENV['SESSION_MAX_SIZE'].to_i,
          ENV['SESSION_THROTTLE_COUNT'].to_i,
          ENV['SESSION_DELAY_TIME'].to_i)
      super(:Reactor)
    end

    ##
    # Intercepts a socket
    def intercept(socket)
      report "Intercepting incoming socket from #{socket.io.addr[-1]}"
      begin
        validate(socket)
        authenticate(socket)
        session = self[:SessionGenerator].generate(socket)
        self[:Manage].register_session(session)
      rescue SocketError
        close_socket(socket)
      end
    end

    private

    ##
    # Validates a socket.
    def validate(socket)
      at = Time.now
      if self[:Manager][:SessionList].key? ip
        unless (at - self[:Manager][:SessionList].last[:InitTime]) < self[:Settings][:DELAY_TIME]
          self[:Component][:API].trigger_events(Strum::Internal::Events::SocketDelayViolationEvent)
          close_socket(socket)
        end
        unless self[:SessionList].size < self[:Settings][:MAX_SIZE]
          self[:Component][:API].trigger_events(Strum::Internal::Events::SocketListOverflow)
          close_socket(socket)
        end
        unless self[:SessionList][ip].size < self[:Settings][:THROTTLE_COUNT]
          self[:Component][:API].trigger_events(Strum::Internal::Events::SocketThrottleEvent)
          close_socket(socket)
        end
      end
    end

    ##
    # Closes a socket and removes it from our Manager's list.
    def close_socket(socket)
      socket.close
      self[:Manager][:SessionList][socket.io.addr[-1]].delete(socket)
    end
    ##
    #
    def authenticate(socket); end

  end
end