# frozen_string_literal: true

# --------------------------------------------- #
# Author: [Patrick W. <"Sickday@pm.me">] ------ #
# InitDate: [3/3/20] ------------------------- #
# --------------------------------------------- #
# Strum
module Strum::Server::Packet
  ##
  #  A Generic Packet object.
  class Template
    include Strum::Internal::Generic
    ##
    # A structure for the internals of our Packet.
    Data = Struct.new(:Header,
                      :Payload,
                      :Terminator)
    ##
    # Creates a new Packet object
    def initialize(data = {op_code: 0, size: -1, headless: false})
      self[:Buffer] = Async::IO::Buffer.new
      self[:Data] = Data.new(
          data[:headless] ? false : Header.new(op_code: data[:op_code], size: data[:size]),
          Payload.new(data[:data] ? data[:data] : nil),
          data[:Terminator] ? Terminator.new(data[:Terminator]) : nil)
    end

    ##
    # Builds this Packet object.
    def build
      validate
      ## Add the Header
      unless self[:Data][:Header] == false
        if self[:Data][:Header][:PacketSize].positive?
          self[:Buffer] << self[:Data][:Header].build(:Byte)
        else
          self[:Buffer] << self[:Data][:Header].build(:Short)
        end
      end

      ## Add the Payload
      self[:Buffer] << self[:Data][:Payload].build

      ## Add the Terminator
      unless self[:Data][:Terminator].nil?
        self[:Buffer] << self[:Data][:Terminator][:Buffer].force_encoding(Encoding::BINARY)
      end

      ## Return buffer
      self[:Buffer]
    end

    ##
    # Validates this packet before building.
    def validate
      unless self[:Data][:Header] == false
        raise Strum::Errors::PacketSizeConflict.new(self) if self[:Data][:Payload][:Buffer].size != self[:Data][:Header][:PacketSize]
      end
      raise Strum::Errors::ClosedGenericBuild if closed?
    end

    ##
    # Writes data to this Packet's buffer.
    def write(template_part, &block)
      if block_given?
        if template_part
          raise Strum::Errors::InvalidTemplatePart.new(template_part) unless [:Header, :Payload, :Terminator].include?(template_part)

          block.call(self[:Data][template_part])
        else
          block.call(self)
        end
      else
        raise Strum::Errors::NilWriteBlock
      end
    end

    alias_method :w!, :write
  end
end